from typing import List

import cq.checker
import cq.utils


class OrangeChecker(cq.checker.Checker):
	NAME = 'orange'
	HELP_LINE = 'run `cq --fix` and make sure that you have pre-commit hooks installed with `pre-commit install`'
	DESCRIPTION = 'Checks whether code complies with the `orange` code style.'
	ORANGE_NOT_INSTALLED = "It looks like you don't have `orange` in this project."

	def run(self, modules: List[str]) -> cq.checker.CheckerResult:
		try:
			orange_output, return_code = cq.utils.run_external_checker('orange', ['--check'] + modules)
		except FileNotFoundError:
			# This means that orange is not available.
			# Projects is probably not yet formatted with orange.
			# We should ignore it for now.
			return cq.checker.CheckerResult(
				checker_name = self.NAME,
				help_line = self.HELP_LINE,
				return_code = 0,
				output = [
					cq.checker.ResultLine(
						file = None,
						line = None,
						message = self.ORANGE_NOT_INSTALLED,
						is_error = False,
					),
				],
			)

		# orange can exit with 1 or 123 if something fails
		if return_code == 0:
			return cq.checker.CheckerResult(
				checker_name = self.NAME,
				help_line = self.HELP_LINE,
				return_code = 0,
				output = [],
			)
		return cq.checker.CheckerResult(
			checker_name = self.NAME,
			help_line = self.HELP_LINE,
			return_code = 1,
			output = [
				cq.checker.ResultLine(
					file = None,
					line = None,
					# Sample output: "1 file would be reformatted."
					message = orange_output.split('\n')[-1],
					is_error = True,
				),
			],
		)



from typing import List

import cq.fixer
import cq.utils


class OrangeFixer(cq.fixer.Fixer):
	NAME = 'orange'
	DESCRIPTION = 'Formats code with the `orange` code style.'

	def run(self, modules: List[str]) -> int:
		return cq.utils.run_external_fixer('orange', modules)



from typing import List

import isort

import cq.checkers.isort_config
import cq.fixer
import cq.utils


class IsortFixer(cq.fixer.Fixer):
	NAME = 'isort'
	DESCRIPTION = 'Sorts imports according to the code style.'

	def run(self, modules: List[str]) -> int:
		for python_file in cq.utils.get_all_python_files(modules):
			isort.file(python_file, config = cq.checkers.isort_config.Config)
		return 0



from typing import List, Optional

import cq.checker
import cq.utils


class PyflakesExtChecker(cq.checker.Checker):
	NAME = 'pyflakes-ext'
	DESCRIPTION = 'Our pyflakes version runner (see https://github.com/qntln/pyflakes-ext)'
	HELP_LINE = 'use `# NOQA` comment for disabling pyflakes on particular line'

	def run(self, modules: List[str]) -> cq.checker.CheckerResult:
		return_code: int = 0
		output: List[cq.checker.ResultLine] = []

		errors, subprocess_return_code = cq.utils.run_external_checker(
			'pyflakes-ext',
			list(cq.utils.get_path_generator(modules)),
		)

		if subprocess_return_code != 0:
			for line in errors.split('\n'):
				if not line:
					continue

				try:
					file_name, line_number, *error_message = line.split(':')
				except ValueError:
					output.append(cq.checker.ResultLine(file = None, line = None, message = line, is_error = True))

					continue

				linenum: Optional[int]
				try:
					linenum = int(line_number)
				except ValueError:
					output.append(cq.checker.ResultLine(file = None, line = None, message = line, is_error = True))

					continue

				output.append(
					cq.checker.ResultLine(
						file = file_name,
						line = linenum,
						message = ':'.join(error_message).lstrip(),
						is_error = True,
					)
				)

			return_code = max(return_code, subprocess_return_code)

		return cq.checker.CheckerResult(
			checker_name = self.NAME,
			help_line = self.HELP_LINE,
			return_code = return_code,
			output = output,
		)



from typing import List, Optional

import cq.checker
import cq.utils


class MypyChecker(cq.checker.Checker):
	NAME = 'mypy'
	DESCRIPTION = 'Run mypy on given modules.'
	HELP_LINE = 'use `# type: ignore` for disabling mypy on particular line'

	def run(self, modules: List[str]) -> cq.checker.CheckerResult:
		options = [
			'--ignore-missing-imports',
			'--warn-unused-ignores',
			'--no-strict-optional',
			'--no-error-summary',
			'--show-error-codes',
		]
		stdout, return_code = cq.utils.run_external_checker('mypy', options + modules)

		output: List[cq.checker.ResultLine] = []

		for output_line in stdout.split('\n'):
			if not output_line:
				continue

			try:
				file, line, *msg = output_line.split(':')
			except ValueError:
				output.append(cq.checker.ResultLine(file = None, line = None, message = output_line, is_error = True))

				continue

			line_number: Optional[int]
			try:
				line_number = int(line)
			except ValueError:
				line_number = None

			output.append(
				cq.checker.ResultLine(
					file = None if line_number is None else file,
					line = line_number,
					message = output_line if line_number is None else ':'.join(msg).lstrip(),
					is_error = True,
				)
			)

		return cq.checker.CheckerResult(
			checker_name = self.NAME,
			help_line = self.HELP_LINE,
			return_code = return_code,
			output = output,
		)



from typing import Set, Tuple
import pathlib
import subprocess

import toml

import cq.checkers.requirements


def _is_poetry_project(path: str) -> bool:
	pyproject_file = pathlib.Path(path) / 'pyproject.toml'
	if not pyproject_file.is_file():
		return False
	pyproject = toml.load(pyproject_file)
	return bool(pyproject.get('tool', {}).get('poetry'))


def autodisable(modules: Tuple[str]) -> Set[str]:
	'''
	Automatically disable checkers based on the environment in which cq runs.
	Rules:
		1. Disable requirements checker for poetry projects
	'''
	disable: Set[str] = set()
	# Disable requirements checker for poetry projects
	try:
		project_root = subprocess.run(['git', 'rev-parse', '--show-toplevel'], capture_output = True, check = True)
		if _is_poetry_project(project_root.stdout.decode('utf-8').strip()):
			disable.add(cq.checkers.requirements.RequirementsvalidatorChecker.NAME)
	except subprocess.CalledProcessError:
		for module in modules:
			if _is_poetry_project(module):
				disable.add(cq.checkers.requirements.RequirementsvalidatorChecker.NAME)
	return disable



import isort


# This is the only way to use isort with custom config file
# and not set project root to cq directory
Config = isort.settings.Config(
	atomic = True,
	ensure_newline_before_comments = True,
	force_grid_wrap = 0,
	from_first = True,
	include_trailing_comma = True,
	indent = 'Tab',
	line_length = 110,
	lines_after_imports = 2,
	multi_line_output = 3,
	order_by_type = True,
	use_parentheses = True,
	extra_standard_library = (
		'typing',
		'ql_types',
		'typing_extensions',
		'mypy_extensions',
		'pyspark.sql.types',
	),
)
